import { NativeCurrency, SerializedToken, Token, Percent, CurrencyAmount, Price, BigintIsh, TradeType } from '@ultron.foundation/swap-sdk-core';
export * from '@ultron.foundation/swap-sdk-core';
import { TxnBuilderTypes } from 'aptos';

declare const APTOS_COIN: "0x1::aptos_coin::AptosCoin";
declare class AptosCoin extends NativeCurrency {
    address: typeof APTOS_COIN;
    structTag: TxnBuilderTypes.StructTag;
    projectLink: string;
    protected constructor(chainId: number);
    private static _aptosCache;
    static onChain(chainId: number): AptosCoin;
    equals(other: Currency): boolean;
    get wrapped(): Coin;
    sortsBefore(other: Currency): boolean;
    get serialize(): SerializedToken;
}

type Currency = AptosCoin | Coin;

declare class Coin extends Token {
    constructor(chainId: number, address: string, decimals: number, symbol: string, name?: string, projectLink?: string);
    sortsBefore(other: Currency): boolean;
    equals(other: Currency): boolean;
}

declare enum ChainId {
    TESTNET = 2,
    MAINNET = 1
}
declare const SWAP_ADDRESS: "0xc7efb4076dbe143cbcd98cfaaa929ecfc8f299203dfff63b95ccb6bfe19850fa";
declare const SWAP_ADDRESS_MODULE: "0xc7efb4076dbe143cbcd98cfaaa929ecfc8f299203dfff63b95ccb6bfe19850fa::swap";
declare const PAIR_RESERVE_TYPE_TAG: "0xc7efb4076dbe143cbcd98cfaaa929ecfc8f299203dfff63b95ccb6bfe19850fa::swap::TokenPairReserve";
declare const PAIR_LP_TYPE_TAG: "0xc7efb4076dbe143cbcd98cfaaa929ecfc8f299203dfff63b95ccb6bfe19850fa::swap::LPToken";
declare const ZERO_PERCENT: Percent;
declare const ONE_HUNDRED_PERCENT: Percent;

declare class Pair {
    readonly liquidityToken: Coin;
    private readonly tokenAmounts;
    static sortToken(tokenA: Currency, tokenB: Currency): [Currency, Currency];
    static getAddress(tokenA: Currency, tokenB: Currency): `0x${string}`;
    static getReservesAddress(tokenA: Currency, tokenB: Currency): `0x${string}`;
    static parseType(type: string): readonly [string, string];
    static getLiquidityToken(tokenA: Currency, tokenB: Currency): Coin;
    constructor(currencyAmountA: CurrencyAmount<Currency>, tokenAmountB: CurrencyAmount<Currency>);
    /**
     * Returns true if the token is either token0 or token1
     * @param token to check
     */
    involvesToken(token: Currency): boolean;
    /**
     * Returns the current mid price of the pair in terms of token0, i.e. the ratio of reserve1 to reserve0
     */
    get token0Price(): Price<Currency, Currency>;
    /**
     * Returns the current mid price of the pair in terms of token1, i.e. the ratio of reserve0 to reserve1
     */
    get token1Price(): Price<Currency, Currency>;
    /**
     * Return the price of the given token in terms of the other token in the pair.
     * @param token token to return price of
     */
    priceOf(token: Currency): Price<Currency, Currency>;
    /**
     * Returns the chain ID of the tokens in the pair.
     */
    get chainId(): number;
    get token0(): Currency;
    get token1(): Currency;
    get reserve0(): CurrencyAmount<Currency>;
    get reserve1(): CurrencyAmount<Currency>;
    reserveOf(token: Currency): CurrencyAmount<Currency>;
    getOutputAmount(inputAmount: CurrencyAmount<Currency>): [CurrencyAmount<Currency>, Pair];
    getInputAmount(outputAmount: CurrencyAmount<Currency>): [CurrencyAmount<Currency>, Pair];
    getLiquidityMinted(totalSupply: CurrencyAmount<Currency>, tokenAmountA: CurrencyAmount<Currency>, tokenAmountB: CurrencyAmount<Currency>): CurrencyAmount<Currency>;
    getLiquidityValue(token: Currency, totalSupply: CurrencyAmount<Currency>, liquidity: CurrencyAmount<Currency>, feeOn?: boolean, kLast?: BigintIsh): CurrencyAmount<Currency>;
}

declare class Route<TInput extends Currency, TOutput extends Currency> {
    readonly pairs: Pair[];
    readonly path: Currency[];
    readonly input: TInput;
    readonly output: TOutput;
    constructor(pairs: Pair[], input: TInput, output: TOutput);
    private _midPrice;
    get midPrice(): Price<TInput, TOutput>;
    get chainId(): number;
}

interface InputOutput<TInput extends Currency, TOutput extends Currency> {
    readonly inputAmount: CurrencyAmount<TInput>;
    readonly outputAmount: CurrencyAmount<TOutput>;
}
declare function inputOutputComparator<TInput extends Currency, TOutput extends Currency>(a: InputOutput<TInput, TOutput>, b: InputOutput<TInput, TOutput>): number;
declare function tradeComparator<TInput extends Currency, TOutput extends Currency, TTradeType extends TradeType>(a: Trade<TInput, TOutput, TTradeType>, b: Trade<TInput, TOutput, TTradeType>): number;
interface BestTradeOptions {
    maxNumResults?: number;
    maxHops?: number;
}
/**
 * Represents a trade executed against a list of pairs.
 * Does not account for slippage, i.e. trades that front run this trade and move the price.
 */
declare class Trade<TInput extends Currency, TOutput extends Currency, TTradeType extends TradeType> {
    /**
     * The route of the trade, i.e. which pairs the trade goes through and the input/output currencies.
     */
    readonly route: Route<TInput, TOutput>;
    /**
     * The type of the trade, either exact in or exact out.
     */
    readonly tradeType: TTradeType;
    /**
     * The input amount for the trade assuming no slippage.
     */
    readonly inputAmount: CurrencyAmount<TInput>;
    /**
     * The output amount for the trade assuming no slippage.
     */
    readonly outputAmount: CurrencyAmount<TOutput>;
    /**
     * The price expressed in terms of output amount/input amount.
     */
    readonly executionPrice: Price<TInput, TOutput>;
    /**
     * The percent difference between the mid price before the trade and the trade execution price.
     */
    readonly priceImpact: Percent;
    /**
     * Constructs an exact in trade with the given amount in and route
     * @param route route of the exact in trade
     * @param amountIn the amount being passed in
     */
    static exactIn<TInput extends Currency, TOutput extends Currency>(route: Route<TInput, TOutput>, amountIn: CurrencyAmount<TInput>): Trade<TInput, TOutput, TradeType.EXACT_INPUT>;
    /**
     * Constructs an exact out trade with the given amount out and route
     * @param route route of the exact out trade
     * @param amountOut the amount returned by the trade
     */
    static exactOut<TInput extends Currency, TOutput extends Currency>(route: Route<TInput, TOutput>, amountOut: CurrencyAmount<TOutput>): Trade<TInput, TOutput, TradeType.EXACT_OUTPUT>;
    static isTradeBetter(tradeA: Trade<Currency, Currency, TradeType> | undefined | null, tradeB: Trade<Currency, Currency, TradeType> | undefined | null, minimumDelta?: Percent): boolean | undefined;
    constructor(route: Route<TInput, TOutput>, amount: TTradeType extends TradeType.EXACT_INPUT ? CurrencyAmount<TInput> : CurrencyAmount<TOutput>, tradeType: TTradeType);
    /**
     * Get the minimum amount that must be received from this trade for the given slippage tolerance
     * @param slippageTolerance tolerance of unfavorable slippage from the execution price of this trade
     */
    minimumAmountOut(slippageTolerance: Percent): CurrencyAmount<TOutput>;
    /**
     * Get the maximum amount in that can be spent via this trade for the given slippage tolerance
     * @param slippageTolerance tolerance of unfavorable slippage from the execution price of this trade
     */
    maximumAmountIn(slippageTolerance: Percent): CurrencyAmount<TInput>;
    /**
     * Given a list of pairs, and a fixed amount in, returns the top `maxNumResults` trades that go from an input token
     * amount to an output token, making at most `maxHops` hops.
     * Note this does not consider aggregation, as routes are linear. It's possible a better route exists by splitting
     * the amount in among multiple routes.
     * @param pairs the pairs to consider in finding the best trade
     * @param nextAmountIn exact amount of input currency to spend
     * @param currencyOut the desired currency out
     * @param maxNumResults maximum number of results to return
     * @param maxHops maximum number of hops a returned trade can make, e.g. 1 hop goes through a single pair
     * @param currentPairs used in recursion; the current list of pairs
     * @param currencyAmountIn used in recursion; the original value of the currencyAmountIn parameter
     * @param bestTrades used in recursion; the current list of best trades
     */
    static bestTradeExactIn<TInput extends Currency, TOutput extends Currency>(pairs: Pair[], currencyAmountIn: CurrencyAmount<TInput>, currencyOut: TOutput, { maxNumResults, maxHops }?: BestTradeOptions, currentPairs?: Pair[], nextAmountIn?: CurrencyAmount<Currency>, bestTrades?: Trade<TInput, TOutput, TradeType.EXACT_INPUT>[]): Trade<TInput, TOutput, TradeType.EXACT_INPUT>[];
    /**
     * Return the execution price after accounting for slippage tolerance
     * @param slippageTolerance the allowed tolerated slippage
     */
    worstExecutionPrice(slippageTolerance: Percent): Price<TInput, TOutput>;
    /**
     * similar to the above method but instead targets a fixed output amount
     * given a list of pairs, and a fixed amount out, returns the top `maxNumResults` trades that go from an input token
     * to an output token amount, making at most `maxHops` hops
     * note this does not consider aggregation, as routes are linear. it's possible a better route exists by splitting
     * the amount in among multiple routes.
     * @param pairs the pairs to consider in finding the best trade
     * @param currencyIn the currency to spend
     * @param nextAmountOut the exact amount of currency out
     * @param maxNumResults maximum number of results to return
     * @param maxHops maximum number of hops a returned trade can make, e.g. 1 hop goes through a single pair
     * @param currentPairs used in recursion; the current list of pairs
     * @param currencyAmountOut used in recursion; the original value of the currencyAmountOut parameter
     * @param bestTrades used in recursion; the current list of best trades
     */
    static bestTradeExactOut<TInput extends Currency, TOutput extends Currency>(pairs: Pair[], currencyIn: TInput, currencyAmountOut: CurrencyAmount<TOutput>, { maxNumResults, maxHops }?: BestTradeOptions, currentPairs?: Pair[], nextAmountOut?: CurrencyAmount<Currency>, bestTrades?: Trade<TInput, TOutput, TradeType.EXACT_OUTPUT>[]): Trade<TInput, TOutput, TradeType.EXACT_OUTPUT>[];
}

interface TradeOptions {
    /**
     * How much the execution price is allowed to move unfavorably from the trade execution price.
     */
    allowedSlippage: Percent;
}
declare abstract class Router {
    static swapCallParameters(trade: Trade<Currency, Currency, TradeType>, options: TradeOptions): ({
        type: string;
    } & {
        function: string;
        type_arguments: string[];
        arguments: any[];
    }) | undefined;
    static addLiquidityParameters(amountX: string, amountY: string, amountXMin: string, amountYMin: string, addressX: string, addressY: string): {
        type: string;
    } & {
        function: string;
        type_arguments: string[];
        arguments: any[];
    };
    static removeLiquidityParameters(liquidityAmount: string, minAmountX: string, minAmountY: string, addressX: string, addressY: string): {
        type: string;
    } & {
        function: string;
        type_arguments: string[];
        arguments: any[];
    };
}

export { AptosCoin, BestTradeOptions, ChainId, Coin, Currency, ONE_HUNDRED_PERCENT, PAIR_LP_TYPE_TAG, PAIR_RESERVE_TYPE_TAG, Pair, Route, Router, SWAP_ADDRESS, SWAP_ADDRESS_MODULE, Trade, TradeOptions, ZERO_PERCENT, inputOutputComparator, tradeComparator };
